/******************************************************************************************************************************************
 *
 * Filename    : CSerialPort.c
 * Project     : CSerialPort
 * Description : Component to access a serial port to send and write characters using Windows API.
 *               This is is not a full-blown component, but a prototype to play around and to understand how the API works.
 *
 * www.sunshine2k.de | www.bastian-molkenthin.de
 *
 * This file is licensed under MIT licence. For further details, see http://www.sunshine2k.de/license.html.
 *
 *****************************************************************************************************************************************/

/******************************************************************************************************************************************
 * INCLUDES
 *****************************************************************************************************************************************/
#include "CSerialPort.h"

/******************************************************************************************************************************************
 * TYPES
 *****************************************************************************************************************************************/
struct CSerialPortStruct
{
  HANDLE  hCommHandle;        /* Handle to COM port */
  DCB     deviceCtrlBlock;    /* Control settings of COM port */
#if !defined ( CSERIALPORT_ENABLE_BLOCKING_TEST_MODE )
  HANDLE  hThread;            /* Handle to reception thread */
  BOOL    threadActive;       /* Flag if reception thread is running */
  HANDLE  hThreadExitEvent;   /* Event to set to exit the reception thread */
#endif
};

/******************************************************************************************************************************************
 * LOCAL FUNCTION DECLARATIONS
 *****************************************************************************************************************************************/

/******************************************************************************************************************************************
 * Name: CSerialPort_ClearTimeouts
 * -------------------------------------
 * Parameter:
 *  - sp: Pointer to SerialCom port object
 * Return value: None.
 * Description: Clears all timeout parameters for the COM port, i.e. disables all timeout handling.
 *****************************************************************************************************************************************/
static void CSerialPort_ClearTimeouts(CSerialPortStruct* sp);

/******************************************************************************************************************************************
 * Name: CSerialPort_SetDefaultTimeouts
 * -------------------------------------
 * Parameter:
 *  - sp: Pointer to SerialCom port object
 * Return value: None.
 * Description: Sets the default timeout values for the COM port. This has the effect that the ReadFile API immediately returns! Either
 *              with the receibed bytes or with no bytes at all if nothing has been receibed yet.
 *****************************************************************************************************************************************/
static void CSerialPort_SetDefaultTimeouts(CSerialPortStruct* sp);

/******************************************************************************************************************************************
 * Name: CSerialPort_ReadThreadFunc
 * -------------------------------------
 * Parameter:
 *  - lpParameter: Pointer to any object
 * Return value: Return value of Thread.
 * Description: Function running in own thread handling the asynchronous reception of data from the COM port. Each time a new character
 *              has been received, the callback function CSerialPort_ReceptionCallout is invoked.
 *****************************************************************************************************************************************/
DWORD WINAPI CSerialPort_ReadThreadFunc(LPVOID lpParameter);

/******************************************************************************************************************************************
 * FUNCTIONS
 *****************************************************************************************************************************************/

/******************************************************************************************************************************************
 * Name: CSerialPort_ClearTimeouts
 *****************************************************************************************************************************************/
static void CSerialPort_ClearTimeouts(CSerialPortStruct* sp)
{
  COMMTIMEOUTS timeouts;

  SecureZeroMemory(&timeouts, sizeof(COMMTIMEOUTS));
  SetCommTimeouts(sp->hCommHandle, &timeouts);
}

/******************************************************************************************************************************************
 * Name: CSerialPort_SetDefaultTimeouts
 *****************************************************************************************************************************************/
static void CSerialPort_SetDefaultTimeouts(CSerialPortStruct* sp)
{
  COMMTIMEOUTS timeouts;

  /* from: https://docs.microsoft.com/de-de/windows/win32/api/winbase/ns-winbase-commtimeouts
  A value of MAXDWORD, combined with zero values for both the ReadTotalTimeoutConstant and ReadTotalTimeoutMultiplier members, 
  specifies that the read operation is to return immediately with the bytes that have already been received, even if no bytes have been received.*/
  timeouts.ReadIntervalTimeout         = MAXDWORD;
  timeouts.ReadTotalTimeoutConstant    = 0;
  timeouts.ReadTotalTimeoutMultiplier  = 0;
  timeouts.WriteTotalTimeoutConstant   = 0;
  timeouts.WriteTotalTimeoutMultiplier = 0;
  SetCommTimeouts(sp->hCommHandle, &timeouts);
}

/******************************************************************************************************************************************
 * Name: CSerialPort_Create
 *****************************************************************************************************************************************/
CSerialPortStruct* CSerialPort_Create(void)
{
  CSerialPortStruct * sp = NULL;
  if ( (sp = malloc(sizeof(CSerialPortStruct))) != NULL )
  {
    sp->hCommHandle = INVALID_HANDLE_VALUE;
  }
  return sp;
}

/******************************************************************************************************************************************
 * Name: CSerialPort_Init
 *****************************************************************************************************************************************/
BOOL CSerialPort_Init(CSerialPortStruct* sp, CSerialPortInputSettings* settings)
{
  HANDLE hCommHdlTmp;
  DWORD flags;

# if defined ( CSERIALPORT_ENABLE_BLOCKING_TEST_MODE )
  flags = 0;
# else
  flags = FILE_FLAG_OVERLAPPED;
# endif

  hCommHdlTmp = CreateFile( settings->PortNumberAddressString,  /* port to be opened */
                            GENERIC_READ | GENERIC_WRITE,       /* read and write access */
                            0,                                  /* no shared */
                            NULL,                               /* no security */
                            OPEN_EXISTING,                      /* open existing port only */
                            flags,                              /* flags */
                            NULL);                              /* no file with attributes to copy */

  if ( hCommHdlTmp == INVALID_HANDLE_VALUE )
  {
    return FALSE;
  }

  /* save communication handle */
  sp->hCommHandle = hCommHdlTmp;

  /* set events that shall be monitored */
  if (!SetCommMask(sp->hCommHandle, EV_RXCHAR | EV_TXEMPTY))
  {
    return FALSE;
  }

  /* get current settings of serial port */
  SecureZeroMemory(&sp->deviceCtrlBlock, sizeof(DCB));
  sp->deviceCtrlBlock.DCBlength = sizeof(DCB);
  if (!GetCommState(sp->hCommHandle, &sp->deviceCtrlBlock))
  {
    return FALSE;
  }

  /* update COM port control settings with provided settings */
  sp->deviceCtrlBlock.BaudRate = settings->BaudRate;
  sp->deviceCtrlBlock.ByteSize = settings->ByteSize;
  sp->deviceCtrlBlock.StopBits = settings->StopBits;
  sp->deviceCtrlBlock.Parity   = settings->Parity;
  if (!SetCommState(sp->hCommHandle, &sp->deviceCtrlBlock))
  {
    return FALSE;
  }

#if !defined ( CSERIALPORT_ENABLE_BLOCKING_TEST_MODE )
  /* set timeouts, this will cause the read operation to return immediately with the received (or no) characters */
  CSerialPort_SetDefaultTimeouts(sp);

  /* Create reception thread */
  sp->hThread = CreateThread(
                  NULL,                             /* default security attributes */
                  0,                                /* default stack size */
                  CSerialPort_ReadThreadFunc,       /* thread function */
                  sp,                               /* argument to thread function is the serialport structure pointer */
                  0,                                /* use default creation flags */
                  NULL);                            /* do not return thread identifier */
  if (sp->hThread == NULL)
  {
    return FALSE;
  }
  sp->threadActive = TRUE;

#endif

  return TRUE;
}

/******************************************************************************************************************************************
 * Name: CSerialPort_GetCommHandle
 *****************************************************************************************************************************************/
HANDLE CSerialPort_GetCommHandle(CSerialPortStruct* sp)
{
  return sp->hCommHandle;
}

/******************************************************************************************************************************************
 * Name: CSerialPort_GetDeviceControlBlock
 *****************************************************************************************************************************************/
DCB CSerialPort_GetDeviceControlBlock(CSerialPortStruct* sp)
{
  return sp->deviceCtrlBlock;
}

/******************************************************************************************************************************************
 * Name: CSerialPort_IsOpen
 *****************************************************************************************************************************************/
BOOL CSerialPort_IsOpen(CSerialPortStruct* sp)
{
  return (sp->hCommHandle != INVALID_HANDLE_VALUE) ? TRUE : FALSE;
}

/******************************************************************************************************************************************
 * Name: CSerialPort_Close
 *****************************************************************************************************************************************/
void CSerialPort_Close(CSerialPortStruct* sp)
{
  if ( sp->hCommHandle != INVALID_HANDLE_VALUE )
  {
    CloseHandle(sp->hCommHandle);
    sp->hCommHandle = INVALID_HANDLE_VALUE;
  }
}

/******************************************************************************************************************************************
 * Name: CSerialPort_Delete
 *****************************************************************************************************************************************/
void CSerialPort_Delete(CSerialPortStruct* sp)
{
  if ( CSerialPort_IsOpen(sp) )
  {
    CSerialPort_Close(sp);
  }

  if ( sp != NULL )
  {
    free(sp);
  }
}

#if !defined ( CSERIALPORT_ENABLE_BLOCKING_TEST_MODE )
/******************************************************************************************************************************************
 * Name: CSerialPort_StopReception
 *****************************************************************************************************************************************/
void CSerialPort_StopReception(CSerialPortStruct* sp)
{
  SetEvent(sp->hThreadExitEvent);
}

/******************************************************************************************************************************************
 * Name: CSerialPort_ReadThreadFunc
 *****************************************************************************************************************************************/
DWORD WINAPI CSerialPort_ReadThreadFunc(LPVOID lpParameter)
{
  OVERLAPPED ovCommEvt;
  DWORD dwEventMask;
  char charBuf[10];
  DWORD numBytesRead, byteReadLoopIdx;
  HANDLE eventHandlesToWaitFor[2];
  DWORD retVal = 0;

  /* store passed pointer to serial port */
  CSerialPortStruct* sp = lpParameter;

  /* create reception event for Comm event */
  SecureZeroMemory(&ovCommEvt, sizeof(ovCommEvt));
  ovCommEvt.hEvent = CreateEvent( 0, TRUE, FALSE, 0);

  while (sp->threadActive)
  {
    /* setup communication event to wait for */
    BOOL ret = WaitCommEvent(sp->hCommHandle, &dwEventMask, &ovCommEvt) ;
    if ( (!ret) && (GetLastError() != ERROR_IO_PENDING))
    {
      sp->threadActive = FALSE;
      retVal = 1;
    }

    /* create event to be used to exit the thread */
    sp->hThreadExitEvent = CreateEvent( 0, TRUE, FALSE, 0);

    /* wait for any event to occur */
    eventHandlesToWaitFor[0] = ovCommEvt.hEvent;
    eventHandlesToWaitFor[1] = sp->hThreadExitEvent;
    DWORD dwWaitObj = WaitForMultipleObjects(2, eventHandlesToWaitFor, FALSE, INFINITE);

    if (dwWaitObj == WAIT_OBJECT_0)
    {
      /* Communication event received */

      /* Transmission completed event - reset it and continue */
      if ( (dwEventMask & EV_TXEMPTY) != 0 )
      {
        ResetEvent ( ovCommEvt.hEvent );
        continue;
      }

      /* Otherwise handle reception event */
      /* Create overlapped event to be used for asynchronous read of the reception data */
      OVERLAPPED ovRead;
      SecureZeroMemory(&ovRead, sizeof(ovRead));
      ovRead.hEvent = CreateEvent( 0, TRUE, FALSE, 0);
      ResetEvent( ovRead.hEvent  );

      BOOL retRead = ReadFile(sp->hCommHandle, charBuf, sizeof(charBuf), &numBytesRead, &ovRead);
      if (retRead)
      {
        if (numBytesRead > 0)
        {
          for (byteReadLoopIdx = 0; byteReadLoopIdx < numBytesRead; byteReadLoopIdx++)
          {
            BOOL calloutRet = CSerialPort_ReceptionCallout(charBuf[byteReadLoopIdx]);
            if (!calloutRet)
            {
              sp->threadActive = FALSE;
            }
          }
        }
      }
      else
      {
        /* Failed to read reception data - shall never occur, stop reception */
        sp->threadActive = FALSE;
        retVal = 2;
      }

      CloseHandle( ovRead.hEvent );
      ResetEvent ( ovCommEvt.hEvent );
    }
    else if (dwWaitObj == WAIT_OBJECT_0 + 1)
    {
      /* Thread quit event received, stop reception thread */
      ResetEvent(sp->hThreadExitEvent);
      CloseHandle(sp->hThreadExitEvent);
      sp->threadActive = FALSE;
    }
  }

  CloseHandle(ovCommEvt.hEvent);

  return retVal;
}
#endif

/******************************************************************************************************************************************
 * Name: CSerialPort_WriteChar
 *****************************************************************************************************************************************/
BOOL CSerialPort_WriteChar(CSerialPortStruct* sp, unsigned char c)
{
  BOOL retVal;
  DWORD numBytesWritten;
#if !defined ( CSERIALPORT_ENABLE_BLOCKING_TEST_MODE )
  OVERLAPPED ov;
#endif

#if defined ( CSERIALPORT_ENABLE_BLOCKING_TEST_MODE )
  retVal = WriteFile(sp->hCommHandle,                     /* port handle */
                     &c,                                  /* data buffer to write */
                     1,                                   /* number of bytes to write */
                     &numBytesWritten,                    /* number of bytes written */
                     NULL  );                             /* no overlapping */
  return retVal;

#else
  /* create event */
  SecureZeroMemory(&ov, sizeof(ov));
  ov.hEvent = CreateEvent( 0, TRUE, FALSE, 0);

  retVal = WriteFile(sp->hCommHandle,                     /* port handle */
    &c,                                                   /* data buffer to write */
    1,                                                    /* number of bytes to write */
    &numBytesWritten,                                     /* number of bytes written */
    &ov);                                                 /* overlapping object */

  if (!retVal)
  {
    WaitForSingleObject(ov.hEvent, INFINITE);
    ResetEvent(ov.hEvent);
  }

  CloseHandle(ov.hEvent);

  return TRUE;
#endif
}

/******************************************************************************************************************************************
 * Name: CSerialPort_WriteBuffer
 *****************************************************************************************************************************************/
BOOL CSerialPort_WriteBuffer(CSerialPortStruct* sp, unsigned char * buf, int bufSize)
{
  BOOL retVal;
  DWORD numBytesWritten;
#if !defined ( CSERIALPORT_ENABLE_BLOCKING_TEST_MODE )
  OVERLAPPED ov;
#endif

#if defined ( CSERIALPORT_ENABLE_BLOCKING_TEST_MODE )
  retVal = WriteFile(sp->hCommHandle,                     /* port handle */
    buf,                                                  /* data buffer to write */
    bufSize,                                              /* number of bytes to write */
    &numBytesWritten,                                     /* number of bytes written */
    NULL  );                                              /* no overlapping */
  return retVal;

#else

  /* create event */
  SecureZeroMemory(&ov, sizeof(ov));
  ov.hEvent = CreateEvent( 0, TRUE, FALSE, 0);

  retVal = WriteFile(sp->hCommHandle,                     /* port handle */
    buf,                                                  /* data buffer to write */
    bufSize,                                              /* number of bytes to write */
    &numBytesWritten,                                     /* number of bytes written */
    &ov);                                                 /* overlapping object */

  if (!retVal)
  {
    WaitForSingleObject(ov.hEvent, INFINITE);
  }

  CloseHandle(ov.hEvent);

  return TRUE;

#endif
}




#if defined ( CSERIALPORT_ENABLE_BLOCKING_TEST_MODE )
/******************************************************************************************************************************************
 * Name: CSerialPort_ReadBuffer_Blocking_ByReadFile
 *****************************************************************************************************************************************/
BOOL CSerialPort_ReadBuffer_Blocking_ByReadFile(CSerialPortStruct* sp, unsigned char * buf, int * bufSize)
{
  BOOL retVal;
  DWORD numBytesRead = 0;

  CSerialPort_ClearTimeouts(sp); /* this will cause the ReadFile API to block */
  /* ReadFile will wait forever until something is received */
  retVal = ReadFile(sp->hCommHandle, buf, *bufSize, &numBytesRead, NULL);
  *bufSize = numBytesRead;

  return retVal;
}

/******************************************************************************************************************************************
* Name: CSerialPort_ReadBuffer_Blocking_ByEvent
*****************************************************************************************************************************************/
BOOL CSerialPort_ReadBuffer_Blocking_ByEvent(CSerialPortStruct* sp, unsigned char * buf, int * bufSize)
{
  BOOL retVal;
  DWORD dwEventMask;
  DWORD numBytesRead = 0;

  CSerialPort_SetDefaultTimeouts(sp);

  /* set event to be monitored */
  if (!SetCommMask(sp->hCommHandle, EV_RXCHAR))
  {
    return FALSE;
  }

  /* wait for monitored event - blocks forever */
  if (!WaitCommEvent(sp->hCommHandle, &dwEventMask, NULL))
  {
    return FALSE;
  }

  /* read received character */
  retVal = ReadFile(sp->hCommHandle, buf, *bufSize, &numBytesRead, NULL);
  *bufSize = numBytesRead;

  return retVal;
}
#endif


