/******************************************************************************************************************************************
 *
 * Filename    : CSerialPort.h
 * Project     : CSerialPort
 * Description : Header file for a demo how to access the COM port using Windows API.
 *
 * www.sunshine2k.de | www.bastian-molkenthin.de
 *
 * This file is licensed under MIT licence. For further details, see http://www.sunshine2k.de/license.html.
 *
 *****************************************************************************************************************************************/
#if !defined ( CSERIALPORT_HEADER )
# define CSERIALPORT_HEADER

/******************************************************************************************************************************************
 * INCLUDES
 *****************************************************************************************************************************************/
# include <windows.h>
# include <tchar.h>

/******************************************************************************************************************************************
 * DEFINES
 *****************************************************************************************************************************************/
/* Set following define to enable the test mode that allows the initialize the COM port in blocking mode (No overlapping used)
   and provides some more APIs that are intended only for playing around and testing (blocking mode) */
/* # define CSERIALPORT_ENABLE_BLOCKING_TEST_MODE */

/******************************************************************************************************************************************
 * TYPES
 *****************************************************************************************************************************************/
/* hides the CSerialPortStruct type from the user code */
typedef struct CSerialPortStruct CSerialPortStruct;

typedef struct CSerialPortInputSettingsTag
{
  DWORD   BaudRate;
  BYTE    PortNumber;
  TCHAR   PortNumberAddressString[20];
  BYTE    ByteSize;
  BYTE    Parity;
  BYTE    StopBits;
} CSerialPortInputSettings;

/******************************************************************************************************************************************
 * FUNCTIONS
 *****************************************************************************************************************************************/

/******************************************************************************************************************************************
 * Name: CSerialPort_Create
 * -------------------------------------
 * Parameter: None
 * - settings: Pointer to settings structure
 * Return value: CSerialPortStruct - pointer to created SerialCom port object. Returns NULL on failure.
 * Description: Initializes the CSerialPort component. Use CSerialPort_Delete() to delete a closed object finally if not used further.
 *****************************************************************************************************************************************/
CSerialPortStruct*    CSerialPort_Create(void);

/******************************************************************************************************************************************
 * Name: CSerialPort_Init
 * -------------------------------------
 * Parameter:
 *  - sp: Pointer to SerialCom port object
 *  - settings: Pointer to port settings to use for initialization
 * Return value: TRUE if initialization succeeded, FALSE otherwise - pointer to created SerialCom port object
 * Description: Opens and intializes a COM port.
 *****************************************************************************************************************************************/
BOOL                  CSerialPort_Init(CSerialPortStruct* sp, CSerialPortInputSettings* settings);

/******************************************************************************************************************************************
 * Name: CSerialPort_IsOpen
 * -------------------------------------
 * Parameter:
 *  - sp: Pointer to SerialCom port object
 * Return value: TRUE if SerialCom port object is initialized and open, otherwise FALSE.
 * Description: Checks if a COM port is open and ready for use.
 *****************************************************************************************************************************************/
BOOL                  CSerialPort_IsOpen(CSerialPortStruct* sp);

/******************************************************************************************************************************************
 * Name: CSerialPort_Close
 * -------------------------------------
 * Parameter:
 *  - sp: Pointer to SerialCom port object
 * Return value: None.
 * Description: Closes an open COM port. Can be reopened with CSerialPort_Init() / CSerialPort_InitDefault().
 *****************************************************************************************************************************************/
void                  CSerialPort_Close(CSerialPortStruct* sp);

/******************************************************************************************************************************************
 * Name: CSerialPort_Delete
 * -------------------------------------
 * Parameter:
 *  - sp: Pointer to SerialCom port object
 * Return value: None.
 * Description: Deletes a closed COM port. The port object must not be used anymore.
 *****************************************************************************************************************************************/
void                  CSerialPort_Delete(CSerialPortStruct* sp);

/******************************************************************************************************************************************
 * Name: CSerialPort_GetCommHandle
 * -------------------------------------
 * Parameter:
 *  - sp: Pointer to SerialCom port object
 * Return value: Communication handle of COM port.
 * Description: Returns the communication handle that can be used in own Windows API calls.
 *****************************************************************************************************************************************/
HANDLE                CSerialPort_GetCommHandle(CSerialPortStruct* sp);

/******************************************************************************************************************************************
 * Name: CSerialPort_GetDeviceControlBlock
 * -------------------------------------
 * Parameter:
 *  - sp: Pointer to SerialCom port object
 * Return value: Device control block.
 * Description: Returns the used device control block that can be used in own Windows API calls.
 *****************************************************************************************************************************************/
DCB                   CSerialPort_GetDeviceControlBlock(CSerialPortStruct* sp);

/******************************************************************************************************************************************
 * Name: CSerialPort_WriteChar
 * -------------------------------------
 * Parameter:
 *  - sp: Pointer to SerialCom port object
 *  - c: Character to send
 * Return value: TRUE if the transmission was successful, FALSE otherwise.
 * Description: Send a single character to the COM port.
 *****************************************************************************************************************************************/
BOOL                  CSerialPort_WriteChar(CSerialPortStruct* sp, unsigned char c);

/******************************************************************************************************************************************
 * Name: CSerialPort_WriteBuffer
 * -------------------------------------
 * Parameter:
 *  - sp: Pointer to SerialCom port object
 *  - buf: Buffer containing the string to send
 *  - bufSize: Length of the string to send
 * Return value: TRUE if the transmission was successful, FALSE otherwise.
 * Description: Send a string to the COM port.
 *****************************************************************************************************************************************/
BOOL                  CSerialPort_WriteBuffer(CSerialPortStruct* sp, unsigned char * buf, int bufSize);

#if !defined ( CSERIALPORT_ENABLE_BLOCKING_TEST_MODE )

/******************************************************************************************************************************************
 * Name: CSerialPort_StopReception
 * -------------------------------------
 * Parameter:
 *  - sp: Pointer to SerialCom port object
 * Return value: None
 * Description: Stops the reception thread. Allowed to be called from another thread.
 *****************************************************************************************************************************************/
void                  CSerialPort_StopReception(CSerialPortStruct* sp);

/******************************************************************************************************************************************
 * Name: CSerialPort_ReceptionCallout
 * -------------------------------------
 * Parameter:
 *  - c: Received character
 * Return value: Return TRUE to continue the reception thread, FALSE to quit reception thread.
 * Description: Callout to be implemented by the user. It is called for each received character on the COM port.
 *****************************************************************************************************************************************/
BOOL                  CSerialPort_ReceptionCallout(char c);
#endif


/* --------------------------------------------------------------------------------------------------------------------------------------*/
#if defined ( CSERIALPORT_ENABLE_BLOCKING_TEST_MODE )
/******************************************************************************************************************************************
 * Name: CSerialPort_ReadBuffer_Blocking_ByReadFile
 * -------------------------------------
 * Parameter sp: Pointer to serial port handle struct
 * Parameter buf: Pointer to buffer in which to store the receive data.
 * Parameter bufSize: Number of bytes expected to be received. The buffer pointed to by buf parameter must have at least the size bufSize.
 *                    After return, holds the number of read bytes.
 * Return value: TRUE if data is successfully read, otherwise FALSE.
 * Description:
 * This function reads the expected number of bytes from the serial port in a blocking way using ReadFile. This means the call to
 * ReadFile() will block and execution will not return from this function until the given number of bytes are received!
 * ReadFile() behaves blocking if all timeout values in COMMTIMEOUTS struct are set to zero.
 * ! This is more an examplary implementation without real benefit. !
 *****************************************************************************************************************************************/
BOOL                  CSerialPort_ReadBuffer_Blocking_ByReadFile(CSerialPortStruct* sp, unsigned char * buf, int * bufSize);

/******************************************************************************************************************************************
* Name: CSerialPort_ReadBuffer_Blocking_ByEvent
* -------------------------------------
* Parameter sp: Pointer to serial port handle struct
* Parameter buf: Pointer to buffer in which to store the receive data.
* Parameter bufSize: Number of bytes expected to be received. The buffer pointed to by buf parameter must have at least the size bufSize.
*                    After return, holds the number of read bytes.
* Return value: TRUE if data is successfully read, otherwise FALSE.
* Description:
* This function reads the expected number of bytes from the serial port in a blocking way. It sets the EV_RXCHAR event to be monitored,
* then WaitCommEvent is called which will block until at least one character is received. The received data is read and returned afterwards
* using ReadFile().
* ! This is more an examplary implementation without real benefit. !
*****************************************************************************************************************************************/
BOOL                  CSerialPort_ReadBuffer_Blocking_ByEvent(CSerialPortStruct* sp, unsigned char * buf, int * bufSize);
# endif

#endif
