/******************************************************************************************************************************************
 *
 * Filename    : InputSettings.c
 * Project     : CSerialPort
 * Description : Reads the predefined COM port settings from file settings.ini which must exist parallel to the executable.
 *
 * www.sunshine2k.de | www.bastian-molkenthin.de
 *
 * This file is licensed under MIT licence. For further details, see http://www.sunshine2k.de/license.html.
 *
 *****************************************************************************************************************************************/

/******************************************************************************************************************************************
 * INCLUDES
 *****************************************************************************************************************************************/
#include "InputSettings.h"
#include "shlwapi.h"

/******************************************************************************************************************************************
 * TYPES
 *****************************************************************************************************************************************/
typedef struct
{
  BOOL    isInitialized;
  TCHAR   settingsFilePath[MAX_PATH];
} InternalInputSettings;

typedef struct
{
  int     baudrateVal;
  TCHAR   baudrateStr[12];
} BaudrateSettings;

typedef struct
{
  TCHAR   parityStr[15];
} ParitySettings;

typedef struct
{
  TCHAR   stopBitStr[15];
} StopbitSettings;

/******************************************************************************************************************************************
 * VARIABLES
 *****************************************************************************************************************************************/
static InternalInputSettings internalSettings;

static BaudrateSettings baudratePredefinedSettings[] = {
  { CBR_110,    _T("CBR_110") },
  { CBR_300,    _T("CBR_300") },
  { CBR_600,    _T("CBR_600") },
  { CBR_1200,   _T("CBR_1200") },
  { CBR_2400,   _T("CBR_2400") },
  { CBR_4800,   _T("CBR_4800") },
  { CBR_9600,   _T("CBR_9600") },
  { CBR_14400,  _T("CBR_14400") },
  { CBR_19200,  _T("CBR_19200") },
  { CBR_38400,  _T("CBR_38400") },
  { CBR_56000,  _T("CBR_56000") },
  { CBR_57600,  _T("CBR_57600") },
  { CBR_115200, _T("CBR_115200") },
  { CBR_128000, _T("CBR_128000") },
  { CBR_256000, _T("CBR_256000") },
};

static ParitySettings parityPredefinedSettings[] = { { _T("NOPARITY")}, { _T("ODDPARITY")}, { _T("EVENPARITY")}, { _T("MARKPARITY")}, { _T("SPACEPARITY")} };
static StopbitSettings stopBitPredefinedSettings[] = { { _T("ONESTOPBIT")}, { _T("ONE5STOPBITS")}, { _T("TWOSTOPBITS")} };

/******************************************************************************************************************************************
 * FUNCTIONS
 *****************************************************************************************************************************************/

/******************************************************************************************************************************************
 * Name: InputSettings_Init
 *****************************************************************************************************************************************/
void InputSettings_Init(CSerialPortInputSettings* settings)
{
  RtlSecureZeroMemory(&internalSettings, sizeof(InternalInputSettings));

  /* get path of executable */
  HMODULE hModule = GetModuleHandleW(NULL);
  TCHAR path[MAX_PATH];
  GetModuleFileName(hModule, path, MAX_PATH);

  /* remove filename from whole path */
  for (int i = _tcslen(path); i > 0; i--)
  {
    if (path[i] == '\\')
    {
      path[i+1] = 0;
      break;
    }
  }

  /* build and store path to settings file */
  _tcscpy_s(internalSettings.settingsFilePath, MAX_PATH, path);
  _tcscat_s(internalSettings.settingsFilePath, MAX_PATH, _T("settings.ini"));

  /* Set default input settings */
  InputSettings_SetDefault(settings);

  internalSettings.isInitialized = TRUE;
}

/******************************************************************************************************************************************
 * Name: InputSettings_ConfigFileExists
 *****************************************************************************************************************************************/
BOOL InputSettings_ConfigFileExists(void)
{
  DWORD fileAttr;

  if (!internalSettings.isInitialized) return FALSE;

  fileAttr = GetFileAttributes(internalSettings.settingsFilePath);
  return (fileAttr != 0xFFFFFFFF);
}

/******************************************************************************************************************************************
 * Name: InputSettings_SetDefault
 *****************************************************************************************************************************************/
void InputSettings_SetDefault(CSerialPortInputSettings* settings)
{
  /* fill the settings with default values */
  settings->PortNumber  = 15;
  _tcscpy_s(settings->PortNumberAddressString, 20, _T("\\\\.\\COM15"));
  settings->BaudRate    = CBR_9600;
  settings->ByteSize    = 8;
  settings->StopBits    = ONESTOPBIT;
  settings->Parity      = NOPARITY;
}

/******************************************************************************************************************************************
 * Name: InputSettings_ConfigFileRead
 *****************************************************************************************************************************************/
BOOL InputSettings_ConfigFileRead(CSerialPortInputSettings* settings)
{
  if (!internalSettings.isInitialized) return FALSE;

  /* COM port */
  settings->PortNumber = GetPrivateProfileInt(_T("SETTINGS"), _T("COMPORT"), 1, internalSettings.settingsFilePath);
  _stprintf_s(settings->PortNumberAddressString, 20, _T("\\\\.\\COM%d"), settings->PortNumber);

  /* Baudrate - no check for validity of value */
  settings->BaudRate = GetPrivateProfileInt(_T("SETTINGS"), _T("BAUDRATE"), CBR_115200, internalSettings.settingsFilePath);
  if ( settings->BaudRate == 0 )
  {
    TCHAR tmpBuf[MAX_PATH];

    if ( GetPrivateProfileString(_T("SETTINGS"), _T("BAUDRATE"), _T("CBR_115200"), tmpBuf, MAXBYTE, internalSettings.settingsFilePath) > 0)
    {
      int numOfSettings = sizeof(baudratePredefinedSettings) / sizeof(BaudrateSettings);
      for (int i = 0; i < numOfSettings; i++)
      {
        if ( _tcscmp(tmpBuf, baudratePredefinedSettings[i].baudrateStr) == 0 )
        {
          settings->BaudRate = baudratePredefinedSettings[i].baudrateVal;
        }
      }
    }
  }

  /* ByteSize */
  settings->ByteSize = GetPrivateProfileInt(_T("SETTINGS"), _T("BYTESIZE"), 8, internalSettings.settingsFilePath);

  /* Parity */
  settings->Parity = GetPrivateProfileInt(_T("SETTINGS"), _T("PARITY"), 0, internalSettings.settingsFilePath);

  /* StopBits */
  settings->StopBits = GetPrivateProfileInt(_T("SETTINGS"), _T("STOPBITS"), 0, internalSettings.settingsFilePath);

  return TRUE;
}

/******************************************************************************************************************************************
 * Name: InputSettings_PrintConfig
 *****************************************************************************************************************************************/
void InputSettings_PrintConfig(CSerialPortInputSettings* settings)
{
  _tprintf(_T("Configured COM port settings:\n"));
  _tprintf(_T("Comport port : %d\n"), settings->PortNumber);
  _tprintf(_T("Baudrate     : %d\n"), settings->BaudRate);
  if (settings->Parity < SPACEPARITY)
  {
    _tprintf(_T("Parity       : %d (%s)\n"), settings->Parity, parityPredefinedSettings[settings->Parity].parityStr);
  }
  else
  {
    _tprintf(_T("Parity       : %d\n"), settings->Parity);
  }
  if (settings->StopBits < TWOSTOPBITS)
  {
    _tprintf(_T("StopBits     : %d (%s)\n"), settings->StopBits, stopBitPredefinedSettings[settings->StopBits].stopBitStr);
  }
  else
  {
    _tprintf(_T("StopBits     : %d\n"), settings->StopBits);
  }
  _tprintf(_T("\n"));
}

