/******************************************************************************************************************************************
 *
 * Filename    : Main.c
 * Project     : CSerialPort
 * Description : Example application for using the CSerialPort component.
 *
 * www.sunshine2k.de | www.bastian-molkenthin.de
 *
 * This file is licensed under MIT licence. For further details, see http://www.sunshine2k.de/license.html.
 *
 *****************************************************************************************************************************************/

#include <stdio.h>
#include <tchar.h>

/******************************************************************************************************************************************
 * INCLUDES
 *****************************************************************************************************************************************/
#include "CSerialPort.h"
#include "InputSettings.h"

/******************************************************************************************************************************************
 * VARIABLES
 *****************************************************************************************************************************************/
CSerialPortStruct* serialPort;      /* Pointer to serial COM port object */
CSerialPortInputSettings settings;  /* Configuration settings */
BOOL receptionActive;

/******************************************************************************************************************************************
 * LOCAL FUNCTION DECLARATIONS
 *****************************************************************************************************************************************/
void PrintHeader(void);

/******************************************************************************************************************************************
 * FUNCTIONS
 *****************************************************************************************************************************************/

/******************************************************************************************************************************************
 * Name: main
 *****************************************************************************************************************************************/
int main()
{
  /* print welcome header */
  PrintHeader();

  /* load COM port settings*/
  InputSettings_Init(&settings);
  if (InputSettings_ConfigFileExists())
  {
    InputSettings_ConfigFileRead(&settings);
  }
  else
  {
    _tprintf(_T("Input settings file does not exist. Using default values:\n"));
  }

  /* print configuration */
  InputSettings_PrintConfig(&settings);

  /* init COM port */
  serialPort = CSerialPort_Create();
  if (serialPort == NULL)
  {
    _tprintf(_T("Failed to initialize COM port!\n"));
     _tprintf(_T("Print any key to quit\n"));
    getchar();
    return;
  }

  BOOL ret = CSerialPort_Init(serialPort, &settings);
  if (ret == FALSE)
  {
    _tprintf(_T("Failed to initialize COM port!\n"));
     _tprintf(_T("Print any key to quit\n"));
    getchar();
    return;
  }

# if defined ( CSERIALPORT_ENABLE_BLOCKING_TEST_MODE )
  _tprintf(_T("Print any key to send char\n"));
   getchar();
  
  char buffer[] = "Hi out there\r\n";
  CSerialPort_WriteBuffer(serialPort, buffer, sizeof(buffer));
  
  _tprintf(_T("Send five characters to COM port to continue\n"));
  _tprintf(_T("Caution: This will really block until five characters are received. No other way to gon...\n"));
  char Readbuffer[32] = { 0 };
  int sizeReadBuffer = 5;
  CSerialPort_ReadBuffer_Blocking_ByReadFile(serialPort, Readbuffer, &sizeReadBuffer);
  _tprintf(_T("Received characters: %hs\n"), Readbuffer);


# else
  _tprintf(_T("Print any key to send single char 'S' to serial port:\n"));
  getchar();

  CSerialPort_WriteChar(serialPort, 'S');
  CSerialPort_WriteChar(serialPort, '\r');
  CSerialPort_WriteChar(serialPort, '\n');
  _tprintf(_T("Char 'S' was send.\n\n"));

  _tprintf(_T("Print any key to send 'Hello world' string:\n"));
  getchar();
  char buffer[] = "Hello world\r\n";
  CSerialPort_WriteBuffer(serialPort, buffer, sizeof(buffer));
  _tprintf(_T("String 'Hello world' was send.\n\n"));

  _tprintf(_T("Wait for reception - all data that is sent on the serial port is received now and printed to console.\n"));
  _tprintf(_T("Either send character 'q' or press key 'q' to quit reception.\n"));
  /* busy waiting for testing - could be done much better in a real application */
  receptionActive = TRUE;
  while (receptionActive)
  {
    if (GetKeyState('Q') & 0x8000)
    {
      CSerialPort_StopReception(serialPort);
      receptionActive = FALSE;
    }
  }
  _tprintf(_T("Reception stopped\n"));

# endif

 _tprintf(_T("Print any key to quit\n"));
  /* remove all characters typed in by the user during the 'receptionActive' loop */
  FlushConsoleInputBuffer(GetStdHandle(STD_INPUT_HANDLE));
  getchar();

  CSerialPort_Close(serialPort);
  CSerialPort_Delete(serialPort);

  return 0;
}

#if !defined ( CSERIALPORT_ENABLE_BLOCKING_TEST_MODE )
/******************************************************************************************************************************************
 * Name: CSerialPort_ReceptionCallout
 *****************************************************************************************************************************************/
BOOL CSerialPort_ReceptionCallout(char c)
{
  _tprintf(_T("Received: %c\n"), c);
  if (c == 'q')
  {
    receptionActive = FALSE;
    return FALSE;
  }
  return TRUE;
}
#endif

/******************************************************************************************************************************************
 * Name: PrintHeader
 *****************************************************************************************************************************************/
void PrintHeader(void)
{
  _tprintf(_T("Serial port test application 1.0\n"));
  _tprintf(_T("www.sunshine2k.de | www.bastian-molkenthin.de\n\n"));
}
